## Christopher Potts and Florian Schwarz
## For "Affective 'this'" (Linguistic Issues in Language Technology)
## Submitted 2009-06-19
## Revised 2009-12-11
##
## The function plotExclamativity() produces the plots and model summaries for the exclamativity experiment.
##
## Arguments:
##  -- Corpus: data.frame in the format of read.csv("exclamativity.csv").
##  -- Phrase: A value from Corpus$Target.
##  -- Title: Optional information for the top of the output plot.
##
##  Output:
##  -- A plot with regression line in red.
##  -- A model summary (to the screen).
exclamativityPlot = function(Corpus=Corpus, Phrase=Phrase, Title="", Regex=FALSE){
  
  ## Get the current phrase.
  PhraseFrame = subset(Corpus, Word==Phrase)

  ## If PhraseFrame$TokenCount is 0, then the LogFreq value is log(0)
  ## = -Inf. This distorts the statistical picture.  We follow the
  ## lead of glm() and smooth the 0s, thereby getting log values of 0.
  for(i in 1:length(PhraseFrame$TokenCount)){
    if (PhraseFrame$TokenCount[i] == 0){
      PhraseFrame$TokenCount[i] = 1
    }
  }

  LogOdds = log(PhraseFrame$TokenCount / (PhraseFrame$RatingWideCount - PhraseFrame$TokenCount))
  DataFrame = data.frame(Rating=PhraseFrame$Rating, Freq=LogOdds)

  ## Quadratic logistic regression.  These functions is in
  ## exclamativeFormulae.R, read in at the top of this script.
  quad = glm(formula = cbind(PhraseFrame$TokenCount, PhraseFrame$RatingWideCount - PhraseFrame$TokenCount) ~ poly(Rating,2,raw=TRUE), family="binomial", data=PhraseFrame)

  roundTo = 3

  ## Quadratic regression numbers.
  #### Linear.
  interceptCoef = round(coef(quad)[1], roundTo)
  linearCoef = round(coef(quad)[2], roundTo)
  linearP = as.data.frame(summary(quad)[12])[2,4]
  ## Never print p = 0, even after rounding.
  if (linearP < 0.001){ linearP = "p < 0.001" } else { linearP = paste("p =", round(linearP, 3)) }
  #### Quadratic.
  quadraticCoef = round(coef(quad)[3], roundTo) 
  quadraticP = round(as.data.frame(summary(quad)[12])[3,4], roundTo)
  ## Never print p = 0, even after rounding.
  if (quadraticP < 0.001){ quadraticP = "p < 0.001" } else { quadraticP = paste("p =", round(quadraticP, 3)) }
    
  ## Get the smallest and largest observed or predicted log-odds.
  QuadPredict = predict(quad)
  AllFreqs.min = min(append(DataFrame$Freq, QuadPredict))
  AllFreqs.max = max(append(DataFrame$Freq, QuadPredict))
  
  ## Create an appropriate Yaxis. This is used only for
  ## plotting. Points involving Yaxis are not displayed.
  Yaxis = DataFrame$Freq
  Yaxis[1] = AllFreqs.min
  Yaxis[2] = AllFreqs.max

  ## Plotting.
  stats = paste("B1 = ", linearCoef, " (", linearP, ")\n", sep="") 
  stats = paste(stats, "B2 = ", quadraticCoef, " (", quadraticP, ")", sep="") 
  plot(DataFrame$Rating, Yaxis,
       type="n",
       xlab="", ylab="Log-odds",
       main=Title,
       sub=stats)
 
  points(DataFrame, pch=19)  ## Empirical data points.
  curve(interceptCoef + linearCoef*x + quadraticCoef*x^2, 1, 5, add=TRUE, col="red")  ## Regression line.

  ## Print a model summary to the screen.
  print("======================================================================")
  print(Phrase)
  print(summary(quad))
  
}

